<?php
namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Order;
use App\Models\OrderItem;
use App\Models\Shop;
use App\Models\Product;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use App\Http\Controllers\AlertsController;

class OrderController extends Controller
{
   public function create(Request $request)
{
    try {
        $data = $request->json()->all();
        $total = 0.0;
        $quantity = 0.0;

        $newestOrder = Order::orderBy('created_at', 'desc')->first();
        if ($newestOrder) {
            $numericPart = (int) substr($newestOrder->ordernumber, 3) + 1;
            $originalLength = strlen(substr($newestOrder->ordernumber, 3));
            $newNumericPart = str_pad($numericPart, $originalLength, '0', STR_PAD_LEFT);
            $nextOrderNumber = 'EXP' . $newNumericPart;
        } else {
            $nextOrderNumber = "EXP0001";
        }

        // Store order items
        foreach ($data['items'] as $item) {
            $orderItem = new OrderItem();
            $orderItem->ordernumber = $nextOrderNumber;
            $orderItem->productid = $item['productID'];
            $orderItem->quantity = $item['quantity'];
            $orderItem->amount = $item['amount'];
            $total += $item['amount'];
            $quantity += $item['quantity'];
            $orderItem->save();

            DB::table('products')
                ->where('id', $item['productID'])
                ->increment('popularity', 1);

            // Get seller ID for the product
            $sellerId = DB::table('products')->where('id', $item['productID'])->value('sellerid');

            // Send alert to the seller
            $sellerAlertData = new Request([
                'userID' => $sellerId,
                'title' => 'New Order Received',
                'message' => 'You have received a new order for your product.',
            ]);
            (new AlertsController())->sendAlert($sellerAlertData);
        }

        // Create a new order
        $order = new Order();
        $order->userId = $data['userId'];
        $order->ordernumber = $nextOrderNumber;
        $order->quantity = $quantity;
        $order->amount = $total;
        $order->status = "Paid";
        $order->checkoutID = $data['checkoutRequestID'];
        $order->orderNotes = $data['orderNotes'];
        $order->save();

        // Send alert to the order creator
        $alertData = new Request([
            'userID' => $data['userId'],
            'title' => 'Order Confirmation',
            'message' => 'Your order has been placed successfully.',
        ]);
        (new AlertsController())->sendAlert($alertData);

        // Send alert to all admins
        $adminIds = DB::table('users')->where('role', 3)->pluck('id');
        foreach ($adminIds as $adminId) {
            $adminAlertData = new Request([
                'userID' => $adminId,
                'title' => 'New Order Placed',
                'message' => 'A new order has been placed.',
            ]);
            (new AlertsController())->sendAlert($adminAlertData);
        }

        return response()->json(['message' => 'Order created successfully'], 200);
    } catch (\Exception $e) {
        Log::error('Error creating order: ' . $e->getMessage());
        return response()->json(['error' => 'Failed to create order. Try again'], 500);
    }
}


    public function getOrdersForCustomer($customerId)
    {
        // Retrieve orders for the specific customer from the database and group by date ascending
        $orders = Order::where('userId', $customerId)
            ->orderBy('created_at')
            ->get()
            ->groupBy(function ($order) {
                return $order->created_at->format('Y-m-d');
            });

        // Return the grouped orders as JSON response with the key "orders"
        return response()->json(['orders' => $orders]);
    }

    public function getAllOrders()
    {
        $orders = Order::join('users', 'orders.userId', '=', 'users.id')
            ->orderBy('orders.created_at')
            ->get([
                'orders.*',
                'users.firstname',
                'users.lastname',
                'users.address',
                'users.phone'
            ])
            ->groupBy(function ($order) {
                return $order->created_at->format('Y-m-d');
            });

        return response()->json(['orders' => $orders]);
    }

    public function orderDetails($orderNumber)
    {
        // Retrieve the user ID from the orders table using the order number
        $order = DB::table('orders')
            ->where('ordernumber', $orderNumber)
            ->select('userId')
            ->first();

        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }

        $userId = $order->userId;

        // Join the products, order_items, orders, users, shops, and product_reviews tables
        $orderItems = DB::table('order_items')
            ->join('products', 'order_items.productid', '=', 'products.id')
            ->join('orders', 'order_items.ordernumber', '=', 'orders.ordernumber')
            ->join('users', 'products.sellerid', '=', 'users.id')
            ->join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->leftJoin('product_reviews', function ($join) use ($userId) {
                $join->on('products.id', '=', 'product_reviews.productid')
                     ->where('product_reviews.userId', '=', $userId);
            })
            ->where('order_items.ordernumber', $orderNumber)
            ->select(
                'products.id as product_id',
                'products.productname as product_name',
                'products.productdescription as description',
                'products.productimage as image',
                'order_items.amount as item_amount',
                'order_items.quantity',
                'orders.amount as order_total',
                'orders.ordernumber',
                'orders.status',
                DB::raw("CONCAT(users.firstname, ' ', users.lastname) as seller_name"),
                'shops.shopname as shop_name',
                DB::raw('IF(product_reviews.id IS NOT NULL, 1, 0) as reviewed')
            )
            ->get();

        if ($orderItems->isEmpty()) {
            return response()->json(['error' => 'No products found for this order'], 404);
        }

        // Extract order details (assuming all items have the same order details)
        $orderDetails = [
            'ordernumber' => $orderItems[0]->ordernumber,
            'order_total' => $orderItems[0]->order_total,
            'status' => $orderItems[0]->status
        ];

        return response()->json([
            'orders' => [
                [
                    'items' => $orderItems->map(function ($item) {
                        return [
                            'product_id' => $item->product_id,
                            'product_name' => $item->product_name,
                            'description' => $item->description,
                            'item_amount' => $item->item_amount,
                            'image' => $item->image,
                            'quantity' => $item->quantity,
                            'seller_name' => $item->seller_name,
                            'shop_name' => $item->shop_name,
                            'reviewed' => $item->reviewed
                        ];
                    }),
                    'order_details' => $orderDetails
                ]
            ]
        ]);
    }

    public function getAdminSalesData()
    {
        $now = Carbon::now();

        // Date ranges
        $todayStart = $now->copy()->startOfDay();
        $weekStart = $now->copy()->startOfWeek();
        $monthStart = $now->copy()->startOfMonth();

        // Lifetime
        $lifetimeSales = Order::sum('amount');
        $lifetimeOrders = Order::where('status', 'completed')->count();
        $lifetimeShops = Shop::count();
        $lifetimeProducts = Product::count();

        // This Month
        $monthSales = Order::where('created_at', '>=', $monthStart)
            ->sum('amount');
        $monthOrders = Order::where('created_at', '>=', $monthStart)
            ->where('status', 'completed')
            ->count();
        $monthShops = Shop::where('created_at', '>=', $monthStart)->count();
        $monthProducts = Product::where('created_at', '>=', $monthStart)->count();

        // This Week
        $weekSales = Order::where('created_at', '>=', $weekStart)
            ->sum('amount');
        $weekOrders = Order::where('created_at', '>=', $weekStart)
            ->where('status', 'completed')
            ->count();
        $weekShops = Shop::where('created_at', '>=', $weekStart)->count();
        $weekProducts = Product::where('created_at', '>=', $weekStart)->count();

        // Today
        $todaySales = Order::where('created_at', '>=', $todayStart)
            ->sum('amount');
        $todayOrders = Order::where('created_at', '>=', $todayStart)
            ->where('status', 'completed')
            ->count();
        $todayShops = Shop::where('created_at', '>=', $todayStart)->count();
        $todayProducts = Product::where('created_at', '>=', $todayStart)->count();

        // Daily sales for this week
        $dailySales = [];
        for ($i = 0; $i < 7; $i++) {
            $dayStart = $weekStart->copy()->addDays($i)->startOfDay();
            $dayEnd = $dayStart->copy()->endOfDay();
            $dailySales[] = Order::whereBetween('created_at', [$dayStart, $dayEnd])
                ->sum('amount');
        }

        // Prepare response
        $data = [
            'lifetime' => [
                'total_sales' => $lifetimeSales,
                'completed_orders' => $lifetimeOrders,
                'created_shops' => $lifetimeShops,
                'created_products' => $lifetimeProducts,
            ],
            'this_month' => [
                'total_sales' => $monthSales,
                'completed_orders' => $monthOrders,
                'created_shops' => $monthShops,
                'created_products' => $monthProducts,
            ],
            'this_week' => [
                'total_sales' => $weekSales,
                'completed_orders' => $weekOrders,
                'created_shops' => $weekShops,
                'created_products' => $weekProducts,
                'daily_sales' => $dailySales,
            ],
            'today' => [
                'total_sales' => $todaySales,
                'completed_orders' => $todayOrders,
                'created_shops' => $todayShops,
                'created_products' => $todayProducts,
            ],
        ];

        return response()->json($data);
    }

public function getSellerDashboard(Request $request, $sellerid)
{
    $sellerId = $sellerid;
    $now = Carbon::now();

    // Date ranges
    $todayStart = $now->copy()->startOfDay();
    $weekStart = $now->copy()->startOfWeek();
    $monthStart = $now->copy()->startOfMonth();

    // Helper function to get the filtered orders based on the seller_id
    $getFilteredOrders = function ($query) use ($sellerId) {
        return $query->whereIn('ordernumber', function ($query) use ($sellerId) {
            $query->select('order_items.ordernumber')
                ->from('order_items')
                ->join('products', 'order_items.productid', '=', 'products.id')
                ->where('products.sellerid', $sellerId);
        });
    };

    // Lifetime
    $lifetimeSales = $getFilteredOrders(Order::query())->sum('amount');
    $lifetimeOrders = $getFilteredOrders(Order::query())->where('status', 'completed')->count();
    $lifetimeShops = Shop::where('sellerid', $sellerId)->count();
    $lifetimeProducts = Product::where('sellerid', $sellerId)->count();

    // This Month
    $monthSales = $getFilteredOrders(Order::where('created_at', '>=', $monthStart))->sum('amount');
    $monthOrders = $getFilteredOrders(Order::where('created_at', '>=', $monthStart))
        ->where('status', 'completed')
        ->count();
    $monthShops = Shop::where('sellerid', $sellerId)->where('created_at', '>=', $monthStart)->count();
    $monthProducts = Product::where('sellerid', $sellerId)->where('created_at', '>=', $monthStart)->count();

    // This Week
    $weekSales = $getFilteredOrders(Order::where('created_at', '>=', $weekStart))->sum('amount');
    $weekOrders = $getFilteredOrders(Order::where('created_at', '>=', $weekStart))
        ->where('status', 'completed')
        ->count();
    $weekShops = Shop::where('sellerid', $sellerId)->where('created_at', '>=', $weekStart)->count();
    $weekProducts = Product::where('sellerid', $sellerId)->where('created_at', '>=', $weekStart)->count();

    // Today
    $todaySales = $getFilteredOrders(Order::where('created_at', '>=', $todayStart))->sum('amount');
    $todayOrders = $getFilteredOrders(Order::where('created_at', '>=', $todayStart))
        ->where('status', 'completed')
        ->count();
    $todayShops = Shop::where('sellerid', $sellerId)->where('created_at', '>=', $todayStart)->count();
    $todayProducts = Product::where('sellerid', $sellerId)->where('created_at', '>=', $todayStart)->count();

    // Daily sales for this week
    $dailySales = [];
    for ($i = 0; $i < 7; $i++) {
        $dayStart = $weekStart->copy()->addDays($i)->startOfDay();
        $dayEnd = $dayStart->copy()->endOfDay();
        $dailySales[] = $getFilteredOrders(Order::whereBetween('created_at', [$dayStart, $dayEnd]))->sum('amount');
    }

    // Prepare response
    $data = [
        'lifetime' => [
            'total_sales' => $lifetimeSales,
            'completed_orders' => $lifetimeOrders,
            'created_shops' => $lifetimeShops,
            'created_products' => $lifetimeProducts,
        ],
        'this_month' => [
            'total_sales' => $monthSales,
            'completed_orders' => $monthOrders,
            'created_shops' => $monthShops,
            'created_products' => $monthProducts,
        ],
        'this_week' => [
            'total_sales' => $weekSales,
            'completed_orders' => $weekOrders,
            'created_shops' => $weekShops,
            'created_products' => $weekProducts,
            'daily_sales' => $dailySales,
        ],
        'today' => [
            'total_sales' => $todaySales,
            'completed_orders' => $todayOrders,
            'created_shops' => $todayShops,
            'created_products' => $todayProducts,
        ],
    ];

    return response()->json($data);
}




    public function updateCheckoutID(Request $request, $orderId)
    {
        try {
            $order = Order::findOrFail($orderId);

            $order->checkoutID = $request->input('checkoutID');
            $order->save();

            return response()->json([
                'message' => 'Checkout ID updated successfully',
                'order' => $order
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'message' => 'Failed to update Checkout ID',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function updateOrderStatus($ordernumber)
    {
        // Define the statuses directly in the controller
        $statusPlaced = 'Placed';
        $statusPaid = 'Paid';
        $statusProcessing = 'Processing';
        $statusDispatch = 'Dispatched';

        // Find the order by ID
        $order = Order::where('ordernumber', $ordernumber)->first();

        // Check if order exists
        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }

        // Update order status based on current status
        switch ($order->status) {
            case $statusPlaced:
            case $statusPaid:
            case $statusProcessing:
                $order->status = $statusDispatch;
                break;
            case $statusDispatch:
                $order->status = $statusProcessing;
                break;
            default:
                // Handle other statuses if needed
                break;
        }

        // Save the updated order
        $order->save();

        return response()->json($order, 200);
    }

    public function completedOrderStatus($ordernumber)
    {
        // Define the statuses directly in the controller
        $statusPlaced = 'Placed';
        $statusPaid = 'Paid';
        $statusProcessing = 'Processing';
        $statusDispatch = 'Dispatched';
        $completed = 'Completed';

        // Find the order by ID
        $order = Order::where('ordernumber', $ordernumber)->first();

        // Check if order exists
        if (!$order) {
            return response()->json(['error' => 'Order not found'], 404);
        }

        // Update order status based on current status
        switch ($order->status) {
            case $statusPlaced:
            case $statusPaid:
            case $statusProcessing:
            case $statusDispatch:
                $order->status = $completed;
                break;
            default:
                // Handle other statuses if needed
                break;
        }

        // Save the updated order
        $order->save();

        return response()->json($order, 200);
    }
}
