<?php
namespace App\Http\Controllers;
use App\Models\ProductExtraImage;
use Illuminate\Http\Request;
use App\Models\Product;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;

class ProductController extends Controller
{
    public function create(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'productname' => 'required',
            'productdescription' => 'required',
            'price' => 'required|numeric|min:0',
            'discountprice' => 'required|numeric|min:0',
            'sellerid' => 'required|numeric|min:0',
            'image' => 'required|image',
            'wholesaleprice' => 'nullable|numeric|min:0',
            'wholesalepricediscounted' => 'nullable|numeric|min:0',
            'wholesaleitems' => 'nullable|integer|min:0',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation Error', 'error' => $validator->errors()], 400);
        }

        $imagePath = $request->file('image')->store('product_images');

        $product = Product::create([
            'productname' => $request->input('productname'),
            'productdescription' => $request->input('productdescription'),
            'price' => $request->input('price'),
            'discountprice' => $request->input('discountprice'),
            'sellerid' => $request->input('sellerid'),
            'productimage' => $imagePath,
            'wholesaleprice' => $request->input('wholesaleprice'),
            'wholesalepricediscounted' => $request->input('wholesalepricediscounted'),
            'wholesaleitems' => $request->input('wholesaleitems'),
        ]);

        return response()->json(['message' => 'Product created successfully', 'product' => $product], 200);
    }

    public function index(Request $request)
    {
        $sellerid = $request->input('sellerid', 0);
        $userid = $request->input('userid', 0);

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call as shop_phone',
                'shops.shopname as shop_name',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.deleted', '=', 0)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call',
                'shops.shopname',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $sellerid == 0 
            ? $productsQuery->where('products.published', 1)->get() 
            : $productsQuery->where('products.sellerid', $sellerid)->where('products.published', 1)->get();

        // Add favorite status if userid is not 0
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function searchProducts(Request $request)
    {
        $sellerid = $request->input('sellerid', 0);
        $userid = $request->input('userid', 0);
        $searchQuery = $request->input('search', '');

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call as shop_phone',
                'shops.shopname as shop_name',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.deleted', '=', 0)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call',
                'shops.shopname',
                'shops.location',
                'product_categories.categoryid'
            );

        // Apply search filter if search query is provided and not empty
        if (!empty($searchQuery)) {
            $productsQuery->where(function($query) use ($searchQuery) {
                $query->where('products.productname', 'LIKE', "%{$searchQuery}%")
                    ->orWhere('products.productdescription', 'LIKE', "%{$searchQuery}%");
            });
        }

        // Apply seller filter if sellerid is provided
        if ($sellerid != 0) {
            $productsQuery->where('products.sellerid', $sellerid);
        }

        // Filter to show only published products
        $productsQuery->where('products.published', 1);

        $products = $productsQuery->get();

        // Add favorite status if userid is not 0
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexSellerProducts(Request $request)
    {
        $sellerid = $request->input('sellerid');
        $userid = $request->input('userid', 0);

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call as shop_phone',
                'shops.shopname as shop_name',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.sellerid', $sellerid)
            ->where('products.deleted', '=', 0)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call',
                'shops.shopname',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $productsQuery->get();

        // Add favorite status if userid is available
        if ($userid !== null) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexAdmin(Request $request)
    {
        $sellerid = $request->input('sellerid', 0);
        $userid = auth()->id();

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call as shop_phone',
                'shops.shopname as shop_name',
                'shops.location as shop_location',
                'product_categories.categoryid'
            )
            ->where('products.deleted', '=', 0); // Added condition for deleted products

        $products = $productsQuery->get();

        return response()->json(['products' => $products], 200);
    }

    public function indexFeatured(Request $request, $userID = null)
    {
        $number = $request->input('number', 0);
        $userid = $request->input('userid', 0);

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname as shop_name',
                'shops.shop_call as shop_phone',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.featured', 1)
            ->where('products.published', 1)
            ->where('products.deleted', '=', 0)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname',
                'shops.shop_call',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $number == 0 ? $productsQuery->get() : $productsQuery->take($number)->get();

        // Add favorite status if userid is available
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexDiscounted(Request $request, $userID = null)
    {
        $number = $request->input('number', 0);
        $userid = $request->input('userid', 0);
        
        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.published',
                'products.featured',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname as shop_name',
                'shops.shop_call as shop_phone',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.published', 1)
            ->whereNotNull('products.discountprice')
            ->where('products.discountprice', '>', 0)
            ->whereColumn('products.discountprice', '<', 'products.price')
            ->where('products.deleted', '=', 0)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.published',
                'products.featured',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname',
                'shops.shop_call',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $number == 0 ? $productsQuery->get() : $productsQuery->take($number)->get();

        // Add favorite status if userid is available
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexTrending(Request $request, $userID = null)
    {
        $number = $request->input('number', 0);
        $userid = $request->input('userid', 0);
        
        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname as shop_name',
                'shops.shop_call as shop_phone',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.published', 1)
            ->where('products.deleted', '=', 0)
            ->orderByDesc('products.popularity')
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname',
                'shops.shop_call',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $number == 0 ? $productsQuery->take(100)->get() : $productsQuery->take($number)->get();

        // Add favorite status if userid is available
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexProductsByCategories(Request $request, $userID = null)
    {
        $categoryID = $request->categoryID;
        $userid = $request->input('userid', 0);
        
        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'product_categories.categoryid',
                'shops.shopname as shop_name',
                'shops.shop_call as shop_phone',
                'shops.location as shop_location',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.published', 1)
            ->where('products.deleted', '=', 0)
            ->where('product_categories.categoryid', $categoryID)
            ->orderByDesc('products.popularity')
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'product_categories.categoryid',
                'shops.shopname',
                'shops.shop_call',
                'shops.location'
            );

        $products = $productsQuery->get();

        // Add favorite status if userid is available
        if ($userid !== 0) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function indexPopularity($userID = null)
    {
        $userid = auth()->id();

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname as shop_name',
                'shops.shop_call as shop_phone',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.published', 1)
            ->where('products.deleted', '=', 0)
            ->where('products.popularity', '>', 0)
            ->orderBy('products.popularity', 'desc')
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shopname',
                'shops.shop_call',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $productsQuery->get();

        // Add favorite status if userid is available
        if ($userid !== null) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function favouriteForUser($userid)
    {
        $favoriteProductIds = DB::table('favorites')
            ->where('userid', $userid)
            ->pluck('productid');

        $productsQuery = Product::join('shops', 'products.sellerid', '=', 'shops.sellerid')
            ->join('product_categories', 'products.id', '=', 'product_categories.productid')
            ->leftJoin('product_reviews', 'products.id', '=', 'product_reviews.productid')
            ->select(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call as shop_phone',
                'shops.shopname as shop_name',
                'shops.location as shop_location',
                'product_categories.categoryid',
                DB::raw('AVG(product_reviews.rating) as average_rating'),
                DB::raw('COUNT(product_reviews.id) as review_count')
            )
            ->where('products.published', 1)
            ->where('products.deleted', '=', 0)
            ->whereIn('products.id', $favoriteProductIds)
            ->groupBy(
                'products.id',
                'products.productname',
                'products.productimage',
                'products.productdescription',
                'products.price',
                'products.discountprice',
                'products.wholesaleprice',
                'products.wholesalepricediscounted',
                'products.wholesaleitems',
                'products.popularity',
                'products.featured',
                'products.published',
                'products.sellerid',
                'products.created_at',
                'products.updated_at',
                'shops.shopimage',
                'shops.shop_call',
                'shops.shopname',
                'shops.location',
                'product_categories.categoryid'
            );

        $products = $productsQuery->get();

        // Add favorite status if userid is available
        if ($userid !== null) {
            $products->each(function ($product) use ($userid) {
                $favorite = DB::table('favorites')
                    ->where('productid', $product->id)
                    ->where('userid', $userid)
                    ->exists();
                $product->favorite = $favorite ? 1 : null;
            });
        } else {
            $products->each(function ($product) {
                $product->favorite = null;
            });
        }

        return response()->json(['products' => $products], 200);
    }

    public function show($id)
    {
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }
        return response()->json(['product' => $product], 200);
    }

    public function update(Request $request, $id)
    {
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->update($request->all());
        return response()->json(['message' => 'Product updated successfully', 'product' => $product], 200);
    }

    public function increasePopularity($id)
    {
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->popularity += 1;
        $product->save();

        return response()->json([], 204);
    }

    public function markAsFeatured($id)
    {
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->featured = 1; // Set featured attribute to 1
        $product->save();

        return response()->json(['message' => 'Product marked as featured', 'product' => $product], 200);
    }

    public function togglePublishedProduct(Request $request, $id)
    {
        $product = Product::find($id);

        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->published = !$product->published;
        $product->save();

        return response()->json(['message' => 'Live status toggled successfully', 'product' => $product], 200);
    }

    public function toggleFeaturedProduct(Request $request, $id)
    {
        $product = Product::find($id);

        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->featured = !$product->featured;
        $product->save();

        return response()->json(['message' => 'Featured toggled successfully', 'product' => $product], 200);
    }

    public function destroy($id)
    {
        $product = Product::find($id);
        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->delete();
        return response()->json(['message' => 'Product deleted successfully'], 200);
    }

    public function createProduct(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'sellerid' => 'required|numeric',
            'productname' => 'required|string|max:255',
            'description' => 'required|string',
            'price' => 'required|numeric',
            'discountedprice' => 'nullable|numeric',
            'wholesaleprice' => 'nullable|numeric',
            'wholesalepricediscounted' => 'nullable|numeric',
            'wholesaleitems' => 'nullable|integer',
            'categoryID' => 'required|numeric',
            'imageFile' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
            'galleryImage1' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
            'galleryImage2' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation Error', 'error' => $validator->errors()], 400);
        }

        try {
            $imagePath = null;
            if ($request->hasFile('imageFile')) {
                $file = $request->file('imageFile');
                $filename = Str::random(20) . '.' . $file->getClientOriginalExtension();
                $file->move(public_path('pimages'), $filename);
                $imagePath = 'pimages/' . $filename;
            }

            $product = Product::create([
                'sellerid' => $request->sellerid,
                'productname' => $request->productname,
                'productdescription' => $request->description,
                'price' => $request->price,
                'discountprice' => $request->discountedprice ?? 0,
                'wholesaleprice' => $request->wholesaleprice ?? 0,
                'wholesalepricediscounted' => $request->wholesalepricediscounted ?? 0,
                'wholesaleitems' => $request->wholesaleitems,
                'productimage' => basename($imagePath),
            ]);

            // Insert into product_categories table
            DB::table('product_categories')->insert([
                'productid' => $product->id,
                'categoryid' => $request->categoryID,
            ]);

            // Handle gallery images
            $galleryImages = ['galleryImage1', 'galleryImage2'];
            foreach ($galleryImages as $galleryImage) {
                if ($request->hasFile($galleryImage)) {
                    $file = $request->file($galleryImage);
                    $filename = Str::random(20) . '.' . $file->getClientOriginalExtension();
                    $file->move(public_path('pimages'), $filename);
                    $imagePath = 'pimages/' . $filename;

                    // Save to product_extra_images table
                    ProductExtraImage::create([
                        'productid' => $product->id,
                        'imagename' => basename($imagePath),
                    ]);
                }
            }

            return response()->json(['message' => 'Product created successfully. The product will be reviewed before being published to buyers.', 'product' => $product], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error creating product', 'error' => $e->getMessage()], 500);
        }
    }

    public function getProductExtraImages($productId)
    {
        try {
            $extraImages = ProductExtraImage::where('productid', $productId)->get();
            return response()->json(['extraImages' => $extraImages], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error fetching extra images', 'error' => $e->getMessage()], 500);
        }
    }

    public function fetchProduct($id, $userID = null)
    {
        $userid = auth()->id();

        // Fetch the product
        $product = DB::table('products')->where('id', $id)->first();
        
        if (!$product) {
            return response()->json(['error' => 'Product not found'], 404);
        }

        // Fetch the category id
        $category = DB::table('product_categories')->where('productid', $id)->first();
        $categoryid = $category ? $category->categoryid : null;

        // Fetch the extra images for the product
        $extraImages = DB::table('product_extra_images')->where('productid', $id)->get();

        // Initialize image paths
        $imagePath1 = null;
        $imagePath2 = null;

        // Check and assign extra images if available
        if ($extraImages->count() > 0) {
            $imagePath1 = $extraImages[0]->imagename;
        }
        if ($extraImages->count() > 1) {
            $imagePath2 = $extraImages[1]->imagename;
        }

        // Debugging: Log the image paths
        Log::info('Image Path 1: ' . $imagePath1);
        Log::info('Image Path 2: ' . $imagePath2);

        // Convert the product to an array and add extra image paths
        $productData = (array) $product;
        $productData['image_path1'] = $imagePath1;
        $productData['image_path2'] = $imagePath2;
        $productData['categoryid'] = $categoryid;

        // Check if the product is a favorite
        if ($userid !== null) {
            $favorite = DB::table('favorites')
                ->where('productid', $product->id)
                ->where('userid', $userid)
                ->exists();
            $productData['favorite'] = $favorite ? 1 : null;
        } else {
            $productData['favorite'] = null;
        }

        return response()->json(['product' => $productData]);
    }

    public function editProduct(Request $request, $id)
    {
        \Log::info('Request Data:', $request->all());
        $validator = Validator::make($request->all(), [
            'sellerid' => 'required|numeric',
            'productname' => 'required|string|max:255',
            'description' => 'required|string',
            'price' => 'required|numeric',
            'discountedprice' => 'nullable|numeric',
            'wholesaleprice' => 'nullable|numeric',
            'wholesalepricediscounted' => 'nullable|numeric',
            'wholesaleitems' => 'nullable|integer',
            'categoryid' => 'required|numeric',
            'imageFile' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
            'galleryImage1' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
            'galleryImage2' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:5048',
        ]);

        if ($validator->fails()) {
            return response()->json(['message' => 'Validation Error', 'errors' => $validator->errors()], 400);
        }

        try {
            $product = Product::findOrFail($id);

            $imagePath = $product->productimage;
            if ($request->hasFile('imageFile')) {
                // Delete old image if exists
                if ($imagePath && file_exists(public_path('pimages/' . $imagePath))) {
                    unlink(public_path('pimages/' . $imagePath));
                }

                $file = $request->file('imageFile');
                $filename = Str::random(20) . '.' . $file->getClientOriginalExtension();
                $file->move(public_path('pimages'), $filename);
                $imagePath = 'pimages/' . $filename;
            }

            $product->update([
                'sellerid' => $request->sellerid,
                'productname' => $request->productname,
                'productdescription' => $request->description,
                'price' => $request->price,
                'discountprice' => $request->discountedprice ?? 0,
                'wholesaleprice' => $request->wholesaleprice ?? 0,
                'wholesalepricediscounted' => $request->wholesalepricediscounted ?? 0,
                'wholesaleitems' => $request->wholesaleitems,
                'productimage' => $imagePath ? basename($imagePath) : $product->productimage,
            ]);

            // Handle gallery images
            $galleryImages = ['galleryImage1', 'galleryImage2'];
            foreach ($galleryImages as $galleryImage) {
                if ($request->hasFile($galleryImage)) {
                    $file = $request->file($galleryImage);
                    $filename = Str::random(20) . '.' . $file->getClientOriginalExtension();
                    $file->move(public_path('pimages'), $filename);
                    $imagePath = 'pimages/' . $filename;

                    // Save to product_extra_images table
                    ProductExtraImage::updateOrCreate(
                        ['productid' => $product->id, 'imagename' => basename($imagePath)],
                        ['imagename' => basename($imagePath)]
                    );
                }
            }

            // Handle category
            $categoryId = $request->categoryid;
            $productCategory = DB::table('product_categories')->where('productid', $product->id)->first();

            if ($productCategory) {
                // Update category
                DB::table('product_categories')
                    ->where('productid', $product->id)
                    ->update(['categoryid' => $categoryId]);
            } else {
                // Insert new category
                DB::table('product_categories')->insert([
                    'productid' => $product->id,
                    'categoryid' => $categoryId,
                ]);
            }

            return response()->json(['message' => 'Product updated successfully.', 'product' => $product], 200);
        } catch (\Exception $e) {
            return response()->json(['message' => 'Error updating product', 'error' => $e->getMessage()], 500);
        }
    }

    public function deleteProduct(Request $request, $id)
    {
        $product = Product::find($id);

        if (!$product) {
            return response()->json(['message' => 'Product not found'], 404);
        }

        $product->deleted = 1;
        $product->save();

        return response()->json(['message' => 'Product deleted successfully'], 200);
    }

    public function toggleFavorite(Request $request, $productId)
    {
        $userId = $request->input('userId');

        // Check if the favorite record exists
        $favorite = DB::table('favorites')
            ->where('productid', $productId)
            ->where('userid', $userId)
            ->first();

        if ($favorite) {
            // If it exists, delete it to unfavorite the product
            DB::table('favorites')
                ->where('productid', $productId)
                ->where('userid', $userId)
                ->delete();
            $isFavorite = 0;
        } else {
            // If it does not exist, create it to favorite the product
            DB::table('favorites')->insert([
                'productid' => $productId,
                'userid' => $userId,
            ]);
            $isFavorite = 1;
        }

        return response()->json(['favorite' => $isFavorite], 200);
    }
}
